<?php

namespace iwaydev\yii2\xhprof;

use XHProfRuns_Default;
use Yii;
use yii\base\Application;
use yii\base\BootstrapInterface;
use yii\base\Component;

class XhprofComponent extends Component implements BootstrapInterface
{
    public bool $enable = true;
    public string $getParameterName = 'profiler';
    public bool $auto = true;

    /**
     * Bootstrap method to be called during application bootstrap stage.
     *
     * @param Application $app the application currently running
     */
    public function bootstrap($app)
    {
        $this->startProfiling($app);
        $app->response->on('beforeSend', [$this, 'stopProfiling']);
    }

    /**
     * @param Application $app
     *
     * @return void
     */
    public function startProfiling(Application $app): void
    {
        if ($this->enabled($app)) {
            xhprof_enable(XHPROF_FLAGS_CPU + XHPROF_FLAGS_MEMORY);
        }
    }

    /**
     * @return void
     */
    public function stopProfiling(): void
    {
        $app = Yii::$app;
        if ($this->enabled($app)) {
            $xhprof_data = xhprof_disable();
            $xhprof_runs = new XHProfRuns_Default();
            $patch = $app->controller->route;
            $type = $app->id . '-' . str_replace('/', '-', $patch);
            $xhprof_runs->save_run($xhprof_data, $type);
        }
    }

    /**
     * @param Application $app
     *
     * @return bool
     */
    private function enabled(Application $app): bool
    {
        return extension_loaded('xhprof') && $this->enable && ($this->auto || $this->manualEnabled($app));
    }

    /**
     * @param Application $app
     *
     * @return bool
     */
    private function manualEnabled(Application $app): bool
    {
        return $app->request->get($this->getParameterName, false) === 'true';
    }
}
